/*============================================================================
  File:     NCIndexCorruption.sql

  Summary:  This script demonstrates fixing non-clustered
			index corruption

  Date:     June 2009

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

/* Setup
USE master;
GO

-- Get the backup from
-- http://www.sqlskills.com/pastConferences.asp
-- Make sure the restore path exists...
RESTORE DATABASE DemoNCIndex FROM
	DISK = 'D:\SQLskills\CorruptionDemoBackups\CorruptDemoNCIndex.bak'
WITH REPLACE, STATS = 10;
GO
*/

-- Run a CHECKDB
DBCC CHECKDB (DemoNCIndex)
WITH NO_INFOMSGS, ALL_ERRORMSGS;
GO


-- Is it just non-clustered indexes?
-- Scan through all the errors looking for index IDs
-- Maybe use WITH TABLERESULTS?
DBCC CHECKDB (DemoNCIndex)
WITH NO_INFOMSGS, ALL_ERRORMSGS, TABLERESULTS;
GO

-- If you wanted to fix them with CHECKDB, it
-- may do single row repairs or rebuild the index,
-- depending on the error.
DBCC CHECKDB (DemoNCIndex, REPAIR_REBUILD)
WITH NO_INFOMSGS, ALL_ERRORMSGS, TABLERESULTS;
GO













-- You need to be in SINGLE_USER mode! Just to
-- fix non-clustered indexes.
--
-- That doesn't make sense. Just rebuild them
-- manually and keep the database online. Try an
-- online rebuild...
USE DemoNCIndex
GO
EXEC sp_HelpIndex 'Customers';
GO

ALTER INDEX - ON Customers REBUILD
WITH (ONLINE = ON);
GO

-- And check again...
DBCC CHECKDB (DemoNCIndex)
WITH NO_INFOMSGS, ALL_ERRORMSGS;
GO

















-- Didn't work! Online index rebuild scans
-- the old index...
-- Offline rebuild doesn't on 2005...
ALTER INDEX CustomerName ON Customers REBUILD;
GO

DBCC CHECKDB (DemoNCIndex)
WITH NO_INFOMSGS, ALL_ERRORMSGS;
GO

-- But on 2008 you may have to drop and recreate
-- the index (or use repair to do it).
USE DemoNCIndex;
GO

DROP INDEX Customers.CustomerName;
GO
/****** Object:  Index [CustomerName]    Script Date: 07/27/2010 10:35:15 ******/
CREATE NONCLUSTERED INDEX [CustomerName] ON [dbo].[Customers] 
(
	[LastName] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
GO