/*============================================================================
  File:     SuspectDatabase.sql

  Summary:  This script creates and fixes the
			DemoSuspect database.

  Date:     June 2009

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE MASTER
GO
IF DATABASEPROPERTYEX ('DemoSuspect', 'Version') > 0
	DROP DATABASE DemoSuspect;
GO
CREATE DATABASE DemoSuspect
GO

USE DemoSuspect;
GO
CREATE TABLE Employees (
	FirstName    VARCHAR (20),
	LastName     VARCHAR (20),
	YearlyBonus	 INT);
GO









INSERT INTO Employees VALUES (
	'Paul', 'Randal', 10000);
INSERT INTO Employees VALUES (
	'Kimberly', 'Tripp', 10000);
GO











-- Simulate an in-flight transaction
BEGIN TRAN;
UPDATE Employees
	SET YearlyBonus = 0
	WHERE LastName = 'Tripp';
GO
CHECKPOINT;
GO

-- Simulate hardware failure with corruption
-- SHUTDOWN WITH NOWAIT in another window and
-- use a hex editor to whack the log file header


-- after shutdown/corruption/startup

USE DemoSuspect;
GO
















-- Uh-oh - maybe detach/reattach will help
SELECT DATABASEPROPERTYEX (
	'DemoSuspect', 'STATUS');
GO

EXEC sp_detach_db 'DemoSuspect';
GO

-- Not allowed on 2008 - let's copy then drop
ALTER DATABASE DemoSuspect SET OFFLINE;
GO

-- Copy... then:
DROP DATABASE DemoSuspect;
GO















-- Try attaching it again
EXEC sp_attach_db @dbname = N'DemoSuspect', 
    @filename1 = N'C:\Program Files\Microsoft SQL Server\MSSQL10.SQL2008\MSSQL\DATA\DemoSuspect.mdf', 
    @filename2 = N'C:\Program Files\Microsoft SQL Server\MSSQL10.SQL2008\MSSQL\DATA\DemoSuspect_log.ldf';
GO












-- Ok - try using the ATTACH_REBUILD_LOG command
CREATE DATABASE DemoSuspect ON
    (NAME = DemoSuspect,
	FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL10.SQL2008\MSSQL\DATA\DemoSuspect.mdf')
FOR ATTACH_REBUILD_LOG
GO















-- Ok - try removing the log file and then
-- using the ATTACH_REBUILD_LOG command
CREATE DATABASE DemoSuspect ON
    (NAME = DemoSuspect,
	FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL10.SQL2008\MSSQL\DATA\DemoSuspect.mdf')
FOR ATTACH_REBUILD_LOG
GO











-- Ok - we're going to have to hack it back into the server

-- Create a dummy database, shutdown the database and swap in
-- the corrupt files, restart and it should be there

CREATE DATABASE DemoSuspect;
GO

-- Check the files are there...

ALTER DATABASE DemoSuspect SET OFFLINE;
GO

-- Now try using the database
ALTER DATABASE DemoSuspect SET ONLINE;
GO

SELECT DATABASEPROPERTYEX (
	'DemoSuspect', 'STATUS');
GO






















-- Let's try EMERGENCY mode repair
ALTER DATABASE DemoSuspect SET EMERGENCY;
GO
DBCC CHECKDB (DemoSuspect, REPAIR_ALLOW_DATA_LOSS)
WITH NO_INFOMSGS, ALL_ERRORMSGS;
GO
















-- Ok - single user mode as well
ALTER DATABASE DemoSuspect SET SINGLE_USER;
GO
DBCC CHECKDB (DemoSuspect, REPAIR_ALLOW_DATA_LOSS)
WITH NO_INFOMSGS, ALL_ERRORMSGS;
GO

















-- Now try again...
USE DemoSuspect;
GO

-- Check the state
SELECT DATABASEPROPERTYEX (
	'DemoSuspect', 'STATUS');
GO

-- What about the data?
SELECT * FROM Employees;
GO


